@@ -15,14 +15,16 @@ pub struct H264Payloader;
 
 pub const STAPA_NALU_TYPE: u8 = 24;
 pub const FUA_NALU_TYPE: u8 = 28;
+pub const FUB_NALU_TYPE: u8 = 29;
 
-pub const FUA_HEADER_SIZE: isize = 2;
+pub const FUA_HEADER_SIZE: usize = 2;
 pub const STAPA_HEADER_SIZE: usize = 1;
 pub const STAPA_NALU_LENGTH_SIZE: usize = 2;
 
 pub const NALU_TYPE_BITMASK: u8 = 0x1F;
 pub const NALU_REF_IDC_BITMASK: u8 = 0x60;
-pub const FUA_START_BITMASK: u8 = 0x80;
+pub const FU_START_BITMASK: u8 = 0x80;
+pub const FU_END_BITMASK: u8 = 0x40;
 
 pub static ANNEXB_NALUSTART_CODE: Bytes = Bytes::from_static(&[0x00, 0x00, 0x00, 0x01]);
 
@@ -42,6 +44,10 @@ fn next_ind(nalu: &Bytes, start: usize) -> (isize, isize) {
 }
 
 fn emit(nalu: &Bytes, mtu: usize, payloads: &mut Vec<Bytes>) {
+    if nalu.is_empty() {
+        return;
+    }
+
     let nalu_type = nalu[0] & NALU_TYPE_BITMASK;
     let nalu_ref_idc = nalu[0] & NALU_REF_IDC_BITMASK;
 
@@ -56,7 +62,7 @@ fn emit(nalu: &Bytes, mtu: usize, payloads: &mut Vec<Bytes>) {
     }
 
     // FU-A
-    let max_fragment_size = mtu as isize - FUA_HEADER_SIZE;
+    let max_fragment_size = mtu as isize - FUA_HEADER_SIZE as isize;
 
     // The FU payload consists of fragments of the payload of the fragmented
     // NAL unit so that if the fragmentation unit payloads of consecutive
@@ -82,7 +88,7 @@ fn emit(nalu: &Bytes, mtu: usize, payloads: &mut Vec<Bytes>) {
     while nalu_data_remaining > 0 {
         let current_fragment_size = std::cmp::min(max_fragment_size, nalu_data_remaining);
         //out: = make([]byte, fuaHeaderSize + currentFragmentSize)
-        let mut out = BytesMut::with_capacity((FUA_HEADER_SIZE + current_fragment_size) as usize);
+        let mut out = BytesMut::with_capacity(FUA_HEADER_SIZE + current_fragment_size as usize);
         // +---------------+
         // |0|1|2|3|4|5|6|7|
         // +-+-+-+-+-+-+-+-+
@@ -160,7 +166,10 @@ impl Payloader for H264Payloader {
 /// H264Packet represents the H264 header that is stored in the payload of an RTP Packet
 #[derive(PartialEq, Debug, Default, Clone)]
 pub struct H264Packet {
+    pub is_avc: bool,
     pub payload: Bytes,
+
+    fua_buffer: Option<BytesMut>,
 }
 
 impl Depacketizer for H264Packet {
@@ -179,7 +188,11 @@ impl Depacketizer for H264Packet {
 
         match nalu_type {
             1..=23 => {
-                payload.put(&*ANNEXB_NALUSTART_CODE);
+                if self.is_avc {
+                    payload.put_u32(packet.len() as u32);
+                } else {
+                    payload.put(&*ANNEXB_NALUSTART_CODE);
+                }
                 payload.put(&*packet.clone());
                 self.payload = payload.freeze();
             }
@@ -197,7 +210,12 @@ impl Depacketizer for H264Packet {
                         )
                         .into());
                     }
-                    payload.put(&*ANNEXB_NALUSTART_CODE);
+
+                    if self.is_avc {
+                        payload.put_u32(nalu_size as u32);
+                    } else {
+                        payload.put(&*ANNEXB_NALUSTART_CODE);
+                    }
                     payload.put(&*packet.slice(curr_offset..curr_offset + nalu_size));
                     curr_offset += nalu_size;
                 }
@@ -209,18 +227,32 @@ impl Depacketizer for H264Packet {
                     return Err(Error::ErrShortPacket.into());
                 }
 
+                if self.fua_buffer.is_none() {
+                    self.fua_buffer = Some(BytesMut::new());
+                }
+
+                if let Some(fua_buffer) = &mut self.fua_buffer {
+                    fua_buffer.put(&*packet.slice(FUA_HEADER_SIZE as usize..));
+                }
+
                 let b1 = packet[1];
-                if b1 & FUA_START_BITMASK != 0 {
+                if b1 & FU_END_BITMASK != 0 {
                     let nalu_ref_idc = b0 & NALU_REF_IDC_BITMASK;
                     let fragmented_nalu_type = b1 & NALU_TYPE_BITMASK;
 
-                    payload.put(&*ANNEXB_NALUSTART_CODE);
-                    payload.put_u8(nalu_ref_idc | fragmented_nalu_type);
-                    payload.put_slice(&*packet.slice(FUA_HEADER_SIZE as usize..));
+                    if let Some(fua_buffer) = self.fua_buffer.take() {
+                        if self.is_avc {
+                            payload.put_u32((fua_buffer.len() + 1) as u32);
+                        } else {
+                            payload.put(&*ANNEXB_NALUSTART_CODE);
+                        }
+                        payload.put_u8(nalu_ref_idc | fragmented_nalu_type);
+                        payload.put(fua_buffer);
+                    }
 
                     self.payload = payload.freeze();
                 } else {
-                    self.payload = packet.slice(FUA_HEADER_SIZE as usize..);
+                    self.payload = Bytes::new();
                 }
             }
             _ => return Err(Error::NaluTypeIsNotHandled(nalu_type).into()),
@@ -229,3 +261,23 @@ impl Depacketizer for H264Packet {
         Ok(())
     }
 }
+
+/// H264PartitionHeadChecker checks H264 partition head
+pub struct H264PartitionHeadChecker;
+
+impl H264PartitionHeadChecker {
+    /// is_partition_head checks if this is the head of a packetized nalu stream.
+    pub fn is_partition_head(packet: &Bytes) -> bool {
+        if packet.len() < 2 {
+            return false;
+        }
+
+        if packet[0] & NALU_TYPE_BITMASK == FUA_NALU_TYPE
+            || packet[0] & NALU_TYPE_BITMASK == FUB_NALU_TYPE
+        {
+            (packet[1] & FU_START_BITMASK) != 0
+        } else {
+            true
+        }
+    }
+}
