@@ -6,7 +6,7 @@ use util::marshal::{Marshal, MarshalSize, Unmarshal};
 
 use anyhow::Result;
 use bytes::{Buf, BufMut};
-use std::time::Duration;
+use std::time::{Duration, SystemTime, UNIX_EPOCH};
 
 pub const ABS_SEND_TIME_EXTENSION_SIZE: usize = 3;
 
@@ -62,7 +62,7 @@ impl Marshal for AbsSendTimeExtension {
 impl AbsSendTimeExtension {
     /// Estimate absolute send time according to the receive time.
     /// Note that if the transmission delay is larger than 64 seconds, estimated time will be wrong.
-    pub fn estimate(&self, receive: Duration) -> Duration {
+    pub fn estimate(&self, receive: SystemTime) -> SystemTime {
         let receive_ntp = unix2ntp(receive);
         let mut ntp = receive_ntp & 0xFFFFFFC000000000 | (self.timestamp & 0xFFFFFF) << 14;
         if receive_ntp < ntp {
@@ -74,15 +74,18 @@ impl AbsSendTimeExtension {
     }
 
     /// NewAbsSendTimeExtension makes new AbsSendTimeExtension from time.Time.
-    pub fn new(send_time: Duration) -> Self {
+    pub fn new(send_time: SystemTime) -> Self {
         AbsSendTimeExtension {
             timestamp: unix2ntp(send_time) >> 14,
         }
     }
 }
 
-pub fn unix2ntp(t: Duration) -> u64 {
-    let u = t.as_nanos() as u64;
+pub fn unix2ntp(st: SystemTime) -> u64 {
+    let u = st
+        .duration_since(UNIX_EPOCH)
+        .unwrap_or_else(|_| Duration::from_secs(0))
+        .as_nanos() as u64;
     let mut s = u / 1_000_000_000;
     s += 0x83AA7E80; //offset in seconds between unix epoch and ntp epoch
     let mut f = u % 1_000_000_000;
@@ -93,7 +96,7 @@ pub fn unix2ntp(t: Duration) -> u64 {
     s | f
 }
 
-pub fn ntp2unix(t: u64) -> Duration {
+pub fn ntp2unix(t: u64) -> SystemTime {
     let mut s = t >> 32;
     let mut f = t & 0xFFFFFFFF;
     f *= 1_000_000_000;
@@ -101,5 +104,7 @@ pub fn ntp2unix(t: u64) -> Duration {
     s -= 0x83AA7E80;
     let u = s * 1_000_000_000 + f;
 
-    Duration::new(u / 1_000_000_000, (u % 1_000_000_000) as u32)
+    UNIX_EPOCH
+        .checked_add(Duration::new(u / 1_000_000_000, (u % 1_000_000_000) as u32))
+        .unwrap_or(UNIX_EPOCH)
 }
